# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""A generic method to get a database connection.

Copyright (c) 2002-2004 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__revision__ = "$Id: db.py,v 1.12 2004/01/05 12:49:15 adim Exp $"

import sys

class UnknownDriver(Exception):
    """raised when a unknown driver is given to get connexion"""

def get_connexion(driver='postgres', host='', database='',
                  user='', password='', quiet=0):
    """return a db connexion according to given arguments
    """
    # POSTGRES ################################################################
    if driver == 'postgres':
        # psycopg #############################################################
        try:
            import psycopg
            if host:
                dburi = 'host=%s  dbname=%s  user=%s' % (host, database, user)
            else:
                dburi = 'dbname=%s  user=%s' % (database, user)
            if password:
                dburi = '%s password=%s' % (dburi, password)
            return psycopg.connect(dburi)
        except ImportError:
            if not quiet:
                print >> sys.stderr, 'psycopg is not available, trying pgdb'
        # pgdb ################################################################
        try:
            import pgdb
            return pgdb.connect(host=host, database=database, user=user,
                                password=password)
        except ImportError:
            if not quiet:
                print >> sys.stderr, 'pgdb is not available, trying pyPgSQL' 
        # pypgsql #############################################################
        try:
            from pyPgSQL import PgSQL
            return PgSQL.connect(host=host or None,
                                 database=database, user=user,
                                 password=password or None)
        except ImportError :
            raise ImportError(_('Unable to import postgres modules'))    

    # MYSQL ###################################################################
    elif driver == 'mysql':
        from pyMySQL import MySQL
        # FIXME: to test !
        return MySQL.connection(host=host, database=database, user=user,
                                password=password)

    else:
        raise UnknownDriver('Unknown driver %s'%driver)
