# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""tests for the astng

Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__revision__ = "$Id: unittest_astng.py,v 1.8 2004/01/14 08:01:44 syt Exp $"

import unittest
import sys
import parser
from os import getcwd
from os.path import join

from logilab.common.astng import builder, astng
from logilab.common.modutils import load_module_from_name

import data
from data import module as test_module

ASTNG_REPR = builder.ASTNGBuilder().build_from_module(test_module)

class ASTNGRepresentationTest(unittest.TestCase):

    def test_module_base_props(self):
        self.assertEqual(ASTNG_REPR.name, 'data.module')
        self.assertEqual(ASTNG_REPR.doc, "test module for astng\n")
        self.assertEqual(ASTNG_REPR.object, test_module)
        self.assertEqual(ASTNG_REPR.parent, None)
        self.assertEqual(ASTNG_REPR.get_frame(), ASTNG_REPR)
        self.assertEqual(ASTNG_REPR.root(), ASTNG_REPR)
        self.assertEqual(ASTNG_REPR.file, join(data.__path__[0], 'module.py'))
        self.assertEqual(ASTNG_REPR.pure_python, 1)
        self.assertEqual(ASTNG_REPR.package, 0)
        self.assert_(not ASTNG_REPR.is_statement())
        self.assertEqual(ASTNG_REPR.get_statement(), ASTNG_REPR)
        self.assertEqual(ASTNG_REPR.node.get_statement(), ASTNG_REPR)

        
    def test_module_locals(self):
        _locals = ASTNG_REPR.locals
        keys = _locals.keys()
        keys.sort()
        self.assert_(_locals is ASTNG_REPR.globals)
        self.assertEqual(len(_locals), 14)
        # __builtins__ is introduced by "from toto import *"
        self.assertEqual(keys, ['MY_DICT', 'YO', 'YOUPI', '__builtins__',
                                '__revision__',
                                'clean', 'cvrtr', 'debuild', 'global_access',
                                'modutils', 'nested_args', 'os', 'redirect', 'spawn'])


    def test_import_methods(self):
        from_ = ASTNG_REPR.locals['modutils']
        self.assertEqual(from_.get_real_name('spawn'), 'Execute')
        self.assertEqual(from_.get_module_object('spawn'), ('logilab.common', 'Execute'))
                         
    def test_function_base_props(self):
        function = ASTNG_REPR.locals['global_access']

        self.assertEqual(function.name, 'global_access')
        self.assertEqual(function.doc, 'function test')
        self.assertEqual(function.object, test_module.global_access)
        self.assert_(function.parent)
        self.assertEqual(function.get_frame(), function)
        self.assertEqual(function.parent.get_frame(), ASTNG_REPR)
        self.assertEqual(function.root(), ASTNG_REPR)
        self.assertEqual(function.argnames, ['key', 'val'])
        self.assertEqual(function.is_method(), 0)


    def test_function_locals(self):
        locals = ASTNG_REPR.locals['global_access'].locals
        keys = locals.keys()
        keys.sort()

        self.assertEqual(len(locals), 4)
        self.assertEqual(keys, ['i', 'key', 'local', 'val'])


    def test_function_navigation(self):
        function = ASTNG_REPR.locals['global_access']
        self.assertEqual(function.get_statement(), function)
        l_sibling = function.previous_sibling()
        self.assert_(isinstance(l_sibling, astng.Assign))
        r_sibling = function.next_sibling()
        self.assert_(isinstance(r_sibling, astng.Class))
        self.assertEqual(r_sibling.name, 'YO')
        last = r_sibling.next_sibling().next_sibling().next_sibling()
        self.assert_(isinstance(last, astng.Assign))
        self.assertEqual(last.next_sibling(), None)
        first = l_sibling.previous_sibling().previous_sibling().previous_sibling().previous_sibling()
        self.assertEqual(first.previous_sibling(), None)
        
    def test_class_base_props(self):
        klass = ASTNG_REPR.locals['YO']

        self.assertEqual(klass.name, 'YO')
        self.assertEqual(klass.doc, 'hehe')
        self.assertEqual(klass.object, test_module.YO)
        self.assert_(klass.parent)
        self.assertEqual(klass.get_frame(), klass)
        self.assertEqual(klass.parent.get_frame(), ASTNG_REPR)
        self.assertEqual(klass.root(), ASTNG_REPR)
        self.assertEqual(klass.basenames, [])


    def test_class_locals(self):
        klass1 = ASTNG_REPR.locals['YO']
        klass2 = ASTNG_REPR.locals['YOUPI']
        locals1 = klass1.locals
        locals2 = klass2.locals
        keys = locals2.keys()
        keys.sort()

        self.assertEqual(locals1.keys(), ['a', '__init__'])
        self.assertEqual(keys, ['__init__', 'class_attr', 'class_method',
                                'method', 'static_method'])


    def test_class_navigation(self):
        klass = ASTNG_REPR.locals['YO']
        self.assertEqual(klass.get_statement(), klass)
        l_sibling = klass.previous_sibling()
        self.assert_(isinstance(l_sibling, astng.Function), l_sibling)
        self.assertEqual(l_sibling.name, 'global_access')
        r_sibling = klass.next_sibling()
        self.assert_(isinstance(r_sibling, astng.Class))
        self.assertEqual(r_sibling.name, 'YOUPI')

    def test_class_members(self):
        klass1 = ASTNG_REPR.locals['YO']
        klass2 = ASTNG_REPR.locals['YOUPI']

        self.assertEqual(klass1.instance_attrs.keys(), ['yo'])
        self.assertEqual(klass2.instance_attrs.keys(), ['member'])


    def test_class_inheritance(self):
        klass1 = ASTNG_REPR.locals['YO']
        klass2 = ASTNG_REPR.locals['YOUPI']

        self.assertEqual(klass1.basenames, [])
        self.assertEqual(klass2.basenames, ['YO'])

    def test_class_ancestor_method(self):
        klass1 = ASTNG_REPR.locals['YO']
        klass2 = ASTNG_REPR.locals['YOUPI']
        anc_klass = klass2.get_ancestor_for_method('__init__')
        self.assert_(isinstance(anc_klass, astng.Class))
        self.assertEqual(anc_klass.name, 'YO')

    def test_class_ancestor_attr(self):
        klass1 = ASTNG_REPR.locals['YO']
        klass2 = ASTNG_REPR.locals['YOUPI']
        anc_klass = klass2.get_ancestor_for_attribute('yo')
        self.assert_(isinstance(anc_klass, astng.Class))
        self.assertEqual(anc_klass.name, 'YO')
        
    def test_method_base_props(self):
        klass2 = ASTNG_REPR.locals['YOUPI']
        method = klass2.locals['method']

        self.assertEqual(method.name, 'method')
        self.assertEqual(method.argnames, ['self'])
        self.assertEqual(method.doc, 'method test')
        self.assertEqual(method.is_method(), 1)
        self.assertEqual(method.is_static_method(), 0)
        self.assertEqual(method.is_class_method(), 0)


    def test_method_locals(self):
        klass2 = ASTNG_REPR.locals['YOUPI']
        method = klass2.locals['method']
        locals = method.locals
        keys = locals.keys()
        keys.sort()
        self.assertEqual(len(locals), 6)
        self.assertEqual(keys, ['MY_DICT', 'a', 'autre', 'b', 'local', 'self'])
        
    def test_static_method_base_props(self):
        klass2 = ASTNG_REPR.locals['YOUPI']
        method = klass2.locals['static_method']

        self.assertEqual(method.argnames, [])
        self.assertEqual(method.is_method(), 1)
        self.assertEqual(method.is_static_method(), 1)
        self.assertEqual(method.is_class_method(), 0)
        
    def test_class_method_base_props(self):
        klass2 = ASTNG_REPR.locals['YOUPI']
        method = klass2.locals['class_method']

        self.assertEqual(method.argnames, ['cls'])
        self.assertEqual(method.is_method(), 1)
        self.assertEqual(method.is_static_method(), 0)
        self.assertEqual(method.is_class_method(), 1)

    def test_func_nested_args(self):
        func = ASTNG_REPR.locals['nested_args']
        self.assertEqual(func.argnames, ['a', ('b', 'c', 'd')])
        local = func.locals.keys()
        local.sort()
        self.assertEqual(local, ['a', 'b', 'c', 'd'])
        self.assertEqual(func.is_method(), 0)
        self.assertEqual(func.is_static_method(), 0)
        self.assertEqual(func.is_class_method(), 0)
       
    def test_as_string(self):
        self.assert_(ASTNG_REPR.as_string())

    def test_build_from_living(self):
        import time
        timeastng = builder.ASTNGBuilder().build_from_module(time)
        self.assert_(timeastng)

    def test_resolve_astng(self):
        yo = ASTNG_REPR.resolve('YO', living=0)
        self.assert_(isinstance(yo, astng.Class))
        self.assertEqual(yo.name, 'YO')
        
        red = ASTNG_REPR.resolve('redirect', living=0)
        self.assert_(isinstance(red, astng.Function))
        self.assertEqual(red.name, 'nested_args')
        
        spawn = ASTNG_REPR.resolve('spawn', living=0)
        self.assert_(isinstance(spawn, astng.Class))
        self.assertEqual(spawn.name, 'Execute')

        method = ASTNG_REPR.locals['YOUPI'].locals['method']
        my_dict = method.resolve('MY_DICT', living=0)
        self.assert_(isinstance(my_dict, astng.Dict))
        
    def test_resolve_object(self):
        yo = ASTNG_REPR.resolve('YO', living=1)
        self.assert_(yo is test_module.YO)
        
        red = ASTNG_REPR.resolve('redirect', living=1)
        self.assert_(red is test_module.nested_args)
        
        from logilab.common import Execute
        spawn = ASTNG_REPR.resolve('spawn', living=1)
        self.assert_(spawn is Execute)

        method = ASTNG_REPR.locals['YOUPI'].locals['method']
        my_dict = method.resolve('MY_DICT', living=1)
        self.assertEqual(my_dict, {})
        
    def test_resolve_default(self):
        yo = ASTNG_REPR.resolve('YO')
        self.assert_(yo is test_module.YO)

    def test_resolve_raise(self):
        from logilab.common.astng import ResolveError
        self.assertRaises(ResolveError, ASTNG_REPR.resolve, 'YOAA', 1)
        self.assertRaises(ResolveError, ASTNG_REPR.resolve, 'YOAA', 0)
        self.assertRaises(ResolveError, ASTNG_REPR.resolve, 'YOAA')

    def test_get_assigned_value(self):
        my_dict = ASTNG_REPR.locals['MY_DICT']
        self.assert_(isinstance(my_dict.get_assigned_value(), astng.Dict))
        a = ASTNG_REPR.locals['YO'].locals['a']
        value = a.get_assigned_value()
        self.assert_(isinstance(value, astng.Const))
        self.assertEqual(value.value, 1)
        
class AncestorsFTC(unittest.TestCase):
    def setUp(self):
        from data import module2
        self._m = builder.ASTNGBuilder().build_from_module(module2)
        
    def test1(self):
        klass = ASTNG_REPR.locals['YOUPI']
        ancs = [a.name for a in astng.ancestors(klass)]
        self.assertEqual(ancs, ['YO'])

    def test2(self):
        klass = self._m.locals['Specialization']
        ancs = [a.name for a in astng.ancestors(klass)]
        self.assertEqual(ancs, ['YOUPI', 'YO', 'YO'])
        

def suite():
    """return the unitest suite"""
    loader = unittest.TestLoader()
    module = sys.modules[__name__]
    if __name__ == '__main__' and len(sys.argv) > 1:
        return loader.loadTestsFromNames(sys.argv[1:], module)
    return loader.loadTestsFromModule(module)

def Run(runner=None):
    """run tests"""
    testsuite = suite()
    if runner is None:
        runner = unittest.TextTestRunner()
        # uncomment next line to write tests results in a file
        #runner.__init__(open('tests.log','w+'))    
    return runner.run(testsuite)
        
if __name__ == '__main__':
    Run()
