"""
unit tests for module fileutils
squeleton generated by /home/syt/cvs_work/logilab/pyreverse/py2tests.py on Sep 08 at 09:1:31


Some file / file path manipulation utilities

"""
__revision__ = "$Id: unittest_fileutils.py,v 1.6 2004/02/17 10:42:05 syt Exp $"

import unittest
import sys
from os.path import join
from os import getcwd, linesep
from logilab.common.fileutils import *

import data
DATA_DIR = data.__path__[0]
NEWLINES_TXT = join(DATA_DIR,'newlines.txt')

class FirstleveldirectoryFunctionTC(unittest.TestCase):

    def test_known_values_first_level_directory(self):
        """return the first level directory of a path"""
        self.assertEqual(first_level_directory('truc/bidule/chouette'), 'truc', None)
        self.assertEqual(first_level_directory('/truc/bidule/chouette'), '/', None)

class RelativepathFunctionTC(unittest.TestCase):

    def test_known_values_relative_path(self):
        """return the first level directory of a path"""
        self.assertEqual(relative_path('toto/index.html', 'index.html'), '../index.html')
        self.assertEqual(relative_path('index.html', 'toto/index.html'), 'toto/index.html')
        self.assertEqual(relative_path('tutu/index.html', 'toto/index.html'), '../toto/index.html')
        self.assertEqual(relative_path('toto/index.html', '/index.html'), '/index.html')
        self.assertEqual(relative_path('/toto/index.html', '/index.html'), '/index.html')
        self.assertEqual(relative_path('index.html', 'index.html'), '')
        self.assertEqual(relative_path('/index.html', '/index.html'), '')

    def test_raise_relative_path(self):
        """return the first level directory of a path, unresolveable cases"""
        self.assertRaises(UnresolvableError, relative_path, '/index.html', 'toto/index.html')
        
class NormReadTC(unittest.TestCase):
    def test_known_values_norm_read(self):
        data = norm_read(NEWLINES_TXT)
        self.assertEqual(data, linesep.join(['# mixed new lines', '1', '2', '3', '']))


class LinesTC(unittest.TestCase):
    def test_known_values_lines(self):
        self.assertEqual(lines(NEWLINES_TXT),
                         ['# mixed new lines', '1', '2', '3'])
        
    def test_known_values_lines_comment(self):
        self.assertEqual(lines(NEWLINES_TXT, comments='#'),
                         ['1', '2', '3'])

class GetByExtTC(unittest.TestCase):
    def test_kv_include(self):
        self.assertEquals(get_by_ext(DATA_DIR, include_exts=('.py',)),
                          ['data/sub/momo.py', 'data/module2.py', 'data/module.py', 'data/__init__.py'])

    def test_kv_exclude(self):
        self.assertEquals(get_by_ext(DATA_DIR, exclude_exts=('.py', '.pyc')),
                          ['data/sub/doc.txt', 'data/newlines.txt'])

def suite():
    """return the unitest suite"""
    loader = unittest.TestLoader()
    testsuite = loader.loadTestsFromModule(sys.modules[__name__])
    return testsuite

def Run(runner=None):
    """run tests"""
    testsuite = suite()
    if runner is None:
        runner = unittest.TextTestRunner()
        # uncomment next line to write tests results in a file
        #runner.__init__(open('tests.log','w+'))    
    return runner.run(testsuite)
        
if __name__ == '__main__':
    Run()
