# Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
Some text manipulation utilities
"""

__revision__ = "$Id: textutils.py,v 1.7 2003/12/17 15:11:20 syt Exp $"

import re
from os import linesep

NORM_SPACES = re.compile('\s+')


def searchall(rgx, data):
    """apply a regexp using "search" until no more match is found
    """
    result = []
    match = rgx.search(data)
    while match is not None:
        result.append(match)
        match = rgx.search(data, match.end())        
    return result

def normalize_text(text, line_len=80, indent=''):
    """normalize a text to display it with a maximum line size and optionaly
    indentation
    """
    text = text.replace(linesep, ' ')
    text = NORM_SPACES.sub(' ', text)
    lines = []
    while text:
        text = text.strip()
        pos = min(len(indent) + len(text) - 1, line_len)
        if pos == line_len:
            pos = pos - len(indent)
            while text[pos] not in (' ', '\t'):
                pos -= 1
            if pos == 0:
                pos = min(len(indent) + len(text) - 1, line_len)
                pos = pos - len(indent)
                while text[pos] not in (' ', '\t'):
                    pos += 1
        lines.append(indent + text[:pos])
        text = text[pos+1:]
    return linesep.join(lines)

def get_csv(string):
    """return a list of string in a csv format"""
    return [word.strip() for word in string.split(',') if word.strip()]

def pretty_match(match, string, underline_char='^'):
    """return a string whith the match location underlined
    """
    start = match.start()
    end = match.end()
    start_line_pos = string.rfind(linesep, 0, start)
    if start_line_pos == -1:
        start_line_pos = 0
        result = []
    else:
        result = [string[:start_line_pos]]
        start_line_pos += len(linesep)
    offset = start - start_line_pos
    underline = ' ' * offset + underline_char * (end - start)
    end_line_pos = string.find(linesep, end)
    if end_line_pos == -1:
        string = string[start_line_pos:]
        result.append(string)
        result.append(underline)
    else:
        end = string[end_line_pos + len(linesep):]
        string = string[start_line_pos:end_line_pos]
        result.append(string)
        result.append(underline)
        result.append(end)
    return linesep.join(result)
